#pragma once

#include "model.h"
#include <ge211.h>

// The radius of each rendered token.
int const token_radius = 50;

// Colors of rendered tokens.
ge211::Color const first_color       = ge211::Color::medium_red();
ge211::Color const second_color      = ge211::Color::medium_blue();
ge211::Color const first_move_color  = first_color.lighten(0.3);
ge211::Color const second_move_color = second_color.lighten(0.3);

// Code for how we interact with the model.
struct Connect4_ui : ge211::Abstract_game
{
    ///
    /// MEMBER FUNCTIONS
    ///
    /// Each of these member functions *overrides* a default implementation
    /// inherited from `ge211::Abstract_game`. For example, the default
    /// implementation of `on_mouse_move` does nothing, but we change it
    /// to keep track of the mouse position.
    ///

    // Displays the current state of the model by adding sprites to the given
    // `Sprite_set`.
    void draw(ge211::Sprite_set&) override;

    // Called by the game engine when the mouse moves.
    void on_mouse_move(ge211::Position) override;

    // Called by the game engine when the mouse button is clicked.
    void on_mouse_down(ge211::Mouse_button, ge211::Position) override;

    // Returns the dimensions that the window should have (based on the grid
    // dimensions).
    ge211::Dimensions initial_window_dimensions() const override;

    // Returns the title to put on the window.
    std::string initial_window_title() const override;

    ///
    /// FIELDS (PRIVATE DATA MEMBERS)
    ///

    // Holds the logical (presentation-independent) state of the game.
    Connect4_model model_;

    // Logical board column where the mouse was last seen.
    int mouse_column_ = -1;

    /// The sprites, for displaying player tokens.
    ge211::Circle_sprite const player1_token_{token_radius, first_color};
    ge211::Circle_sprite const player2_token_{token_radius, second_color};
    ge211::Circle_sprite const move1_token_{token_radius, first_move_color};
    ge211::Circle_sprite const move2_token_{token_radius, second_move_color};
};
