#include "model.h"
#include <catch.h>
#include <stdexcept>

TEST_CASE("can create default Connect Four")
{
    Connect4_model model;
}

TEST_CASE("game parameters")
{
    CHECK( Connect4_model::k == 4 );
    CHECK( Connect4_model::m == 7 );
    CHECK( Connect4_model::n == 6 );
}

TEST_CASE("can move")
{
    const Player red = Player::first,
                 blu = Player::second,
                 mt  = Player::neither;
    using Column = Connect4_model::column_t;

    Connect4_model c4;

    CHECK( c4.get_turn() == red );
    CHECK( c4.get_winner() == mt );
    c4.place_token(0);
    CHECK( *c4.get_column(0) == Column{red} );

    CHECK( c4.get_turn() == blu );
    c4.place_token(1);
    CHECK( *c4.get_column(1) == Column{blu} );

    CHECK( c4.get_turn() == red );
    c4.place_token(1);
    CHECK( *c4.get_column(1) == Column{blu, red} );

    CHECK( c4.get_turn() == blu );
    c4.place_token(2);
    CHECK( *c4.get_column(2) == Column{blu} );

    CHECK( c4.get_turn() == red );
    c4.place_token(3);
    CHECK( *c4.get_column(3) == Column{red} );

    CHECK( c4.get_turn() == blu );
    c4.place_token(2);
    CHECK( *c4.get_column(2) == Column{blu, blu} );

    CHECK( c4.get_turn() == red );
    c4.place_token(2);
    CHECK( *c4.get_column(2) == Column{blu, blu, red} );

    CHECK( c4.get_turn() == blu );
    c4.place_token(3);
    CHECK( *c4.get_column(3) == Column{red, blu} );

    CHECK( c4.get_turn() == red );
    c4.place_token(2);
    CHECK( *c4.get_column(2) == Column{blu, blu, red, red} );

    CHECK( c4.get_turn() == blu );
    c4.place_token(3);
    CHECK( *c4.get_column(3) == Column{red, blu, blu} );

    CHECK( c4.get_turn() == red );
    c4.place_token(3);
    CHECK( *c4.get_column(3) == Column{red, blu, blu, red} );

    CHECK( c4.get_turn() == mt );
    CHECK( c4.get_winner() == red );

    CHECK_THROWS_AS(c4.place_token(0), std::logic_error);
}

TEST_CASE("full column throws")
{
    const Player red = Player::first,
            blu = Player::second;
    using Column = Connect4_model::column_t;

    Connect4_model c4;

    for (int i = 0; i < c4.n; ++i)
        c4.place_token(2);

    CHECK_THROWS_AS(c4.place_token(2), std::invalid_argument);

    c4.place_token(0);

    CHECK( *c4.get_column(0) == Column{red} );
    CHECK( *c4.get_column(2) == Column{red, blu, red, blu, red, blu} );

    CHECK_THROWS_AS(c4.place_token(2), std::invalid_argument);
}
